/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */
 
package campus;


/**
 *  Contains information about an access point.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: AccessPoint.java,v 1.2 2003/09/12 20:44:22 glchen Exp $
 */
public class AccessPoint 
{
    protected String _name;
    protected double _x;
    protected double _y;
    protected double _z;
    protected int _floor;
    public String _building;
    public String _antenna;
    public String _model; 
    public String _info;    // description about where it is located
    public String _channel;
    public String _power;
    public String _ipaddress;
    public String _gateway;
    public String _macaddress;

    //=======================================================================
    // Getters and Setters.
    //=======================================================================

    public void setName(String name) { _name = name; }
    public String getName() { return _name; }

    public void setX(double x) { _x = x; }
    public double getX() { return _x; }

    public void setY(double y) { _y = y; }
    public double getY() { return _y; }

    public void setZ(double z) { _z = z; }
    public double getZ() { return _z; }

    public void setFloor(int floor) { _floor = floor; }
    public int getFloor() { return _floor; }

    public void setBuilding(String building) { _building = building; }
    public String getBuilding() { return _building; }

    public void setAntenna(String antenna) { _antenna = antenna; }
    public String getAntenna() { return _antenna; }

    public void setModel(String model) { _model = model; }
    public String getModel() { return _model; }
    
    public void setInfo(String info) { _info = info; }
    public String getInfo() { return _info; }

    public void setChannel(String channel) { _channel = channel; }
    public String getChannel() { return _channel; }

    public void setPower(String power) { _power = power; }
    public String getPower() { return _power; }

    public void setIpAddress(String ipaddress) { _ipaddress = ipaddress; }
    public String getIpAddress() { return _ipaddress; }

    public void setGateway(String gateway) { _gateway = gateway; }
    public String getGateway() { return _gateway; }

    public void setMacAddress(String macaddress) { _macaddress = macaddress; }
    public String getMacAddress() { return _macaddress; }


    //=======================================================================
    // Public interface methods.
    //=======================================================================

    /** Return a string representation of this access point. */
    public String toString()
    {
        StringBuffer sb = new StringBuffer();
        sb.append("[name=" + _name + ",");
        sb.append("x=" + Double.toString(_x) + ",");
        sb.append("y=" + Double.toString(_y) + ",");
        sb.append("z=" + Double.toString(_z) + ",");
        sb.append("floor=" + Integer.toString(_floor) + ",");
        sb.append("building=" + _building + ",");
        sb.append("antenna=" + _antenna + ",");
        sb.append("model=" + _model + ",");
        sb.append("info=" + _info + ",");
        sb.append("channel=" + _channel + ",");
        sb.append("power=" + _power + ",");
        sb.append("ip=" + _ipaddress + ",");
        sb.append("gateway=" + _gateway + ",");
        sb.append("mac=" + _macaddress + "]");
        return sb.toString();
    }

    /** Convert an AP name to a code representing one building. */
    public static String getBuildingCode(String apname)
    {
        String ret = apname.replaceAll("-wb", "");
        ret = ret.replaceAll("[0-9]*-ap", "");
        if (ret.endsWith("-"))
        {
            ret = ret.substring(0, ret.length()-1);
        }
        return ret;
    }
}
