/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package campus.compass;


import com.meterware.httpunit.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Hashtable;
import campus.Location;


/**
 *  Given an IP address, determine its geo location using a Web site.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: GeobytesSource.java,v 1.4 2003/09/12 18:09:06 glchen Exp $
 */
public class GeobytesSource 
{
    public static final long VISIT_LIMIT = 120000;  // 2 minute
    
    protected static long _lastVisited = 0;
    protected static Hashtable _mapping = new Hashtable();
    
    
    //=======================================================================
    // Consructors.
    //=======================================================================
    
    public GeobytesSource()
    {
    }
    

    //=======================================================================
    // Public interface methods.
    //=======================================================================

    public synchronized static Location getLocationByIp(String ip)
        throws Exception
    {
        // check mapping table first
        Location ret = (Location) _mapping.get(ip);
        if (ret != null)
        {
            return ret;
        }

        // check timestamp to prevent too frequent visits
        if (System.currentTimeMillis() - _lastVisited < VISIT_LIMIT)
        {
            throw new Exception("Busy, come back later.");
        }
        
        // use IP locator from GeoBytes Inc.
        String ipLocator = "http://www.geobytes.com/IpLocator.htm";
        WebConversation wc = new WebConversation();
        WebResponse wp = wc.getResponse(ipLocator);
        WebForm[] forms = wp.getForms();
        if (forms.length == 0)
        {
            throw new Exception("Busy, come back later.");
        }
        WebForm form = forms[0];
        form.setParameter("ipaddress", ip);
        wp = form.submit();

        // have to manually parse the result
        ret = new Location();
        BufferedReader in = new BufferedReader(
            new InputStreamReader(wp.getInputStream()));
        String line = in.readLine();
        while (line != null)
        {
            if (line.indexOf(">Country <input") != -1)
            {
                String c = getValueFromLine(line);
                line = CountryCode.getCountryCode(c);
                if (line == null)
                {
                    ret.setCountry(c);
                }
                else
                {
                    ret.setCountry(line);
                }
            }
            else if (line.indexOf(">Region <input") != -1)
            {
                ret.setState(
                    ZipCodeSource.getStateAbbr(getValueFromLine(line)));
            }
            else if (line.indexOf(">City <input") != -1)
            {
                ret.setCity(getValueFromLine(line));
            }
            else if (line.indexOf(">Certainty <input") != -1)
            {
                ret.setCertainty(Integer.parseInt(getValueFromLine(line)));
            }
            line = in.readLine();
        }

        // record the mapping and return the result
        _mapping.put(ip, ret);
        _lastVisited = System.currentTimeMillis();
        return ret;
    }


    private static String getValueFromLine(String line)
    {
        int p1 = line.indexOf("value=\"");
        int p2 = line.indexOf("\"", p1 + 8);
        return line.substring(p1 + 7, p2);
    }
}
