/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package campus.compass;


import java.io.FileReader;
import java.io.BufferedReader;
import java.util.Hashtable;

import solar.util.SystemUtil;
import campus.Location;


/**
 *  Given an IP address, determine its geo location using whois command.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: WhoisSource.java,v 1.4 2003/09/12 18:09:07 glchen Exp $
 */
public class WhoisSource 
{
    protected static Hashtable _mapping = new Hashtable();
    
    //=======================================================================
    // Consructors.
    //=======================================================================
    
    public WhoisSource()
    {
    }
    

    //=======================================================================
    // Public interface methods.
    //=======================================================================

    public synchronized static Location getLocationByIp(String ip)
        throws Exception
    {
        // check mapping table first
        Location ret = (Location) _mapping.get(ip);
        if (ret != null)
        {
            return ret;
        }

        // create tmp file
        String tmpfile = "/tmp/whois-" + 
            Long.toString(System.currentTimeMillis());
        
        Process whois = 
            SystemUtil.command("whois " + ip + " > " + tmpfile);
        whois.waitFor();
        BufferedReader in = 
            new BufferedReader(new FileReader(tmpfile));

        // parse the tmp file to get location information
        ret = new Location();
        String line = in.readLine();
        while (line != null)
        {
            if (line.indexOf("City:") != -1)
            {
                ret.setCity(line.substring(6).trim());
            }
            else if (line.indexOf("StateProv:") != -1)
            {
                ret.setState(line.substring(11).trim());
            }
            else if (line.indexOf("PostalCode:") != -1)
            {
                ret.setZipCode(line.substring(12).trim());
            }
            else if (line.indexOf("Country:") != -1)
            {
                ret.setCountry(line.substring(10).trim());
            }                
            line = in.readLine();
        }
        ret.setCertainty(50);
        in.close();
        // remove the tmp file
        SystemUtil.command("rm -f " + tmpfile);

        // record the mapping and return the result
        _mapping.put(ip, ret);
        return ret;
    }
}
