/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package campus.sysnort;


import java.util.List;
import java.util.Vector;
import java.util.Iterator;
import java.util.Hashtable;

import solar.util.NetUtil;


/**
 *  A Sysnort configuration contains a set of triggers parsed from the
 *  configuration file.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: SysnortConfig.java,v 1.3 2003/08/19 19:43:46 glchen Exp $
 */
class SysnortConfig
{
    protected Hashtable _config;

    
    //=======================================================================
    // Constructors.
    //=======================================================================

    public SysnortConfig()
    {
        _config = new Hashtable();
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================
    
    public TriggerConfig getTrigger(String name)
    {
        return (TriggerConfig) _config.get(name);
    }


    public void putTrigger(String name, TriggerConfig trigger)
    {
        _config.put(name, trigger);
    }


    public List loadTriggers()
        throws Exception
    {
        List ret = new Vector();
        Iterator itr = _config.keySet().iterator();
        while (itr.hasNext())
        {
            String name = (String) itr.next();
            TriggerConfig tc = getTrigger(name);
            Trigger trigger = (Trigger) 
                NetUtil.loadObject(tc.getCodebase(), tc.getClassname());
            trigger.init(tc.getArguments());
            trigger.setName(tc.getName());
            ret.add(trigger);
        }
        return ret;
    }

}
