/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */


package campus.sysnort;


import serp.util.Options;

import solar.api.Event;


/**
 *  Abstract class for Sysnort trigger.  A trigger has a condition and
 *  an action.  Sysnort checks an arriving event against the condition
 *  and calls the action if the condition is satisfied.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: Trigger.java,v 1.4 2003/08/22 01:06:46 glchen Exp $
 */
public abstract class Trigger 
{
    protected Object _actionObject;
    protected String _name;
    
    //=======================================================================
    // Public interface methods.
    //=======================================================================

    /** Init the trigger with customized arguments in configuration. */
    public void init(Options opts)
        throws Exception
    {
        // default to do nothing
    }


    /** Set the name of this trigger. */
    public void setName(String name)
    {
        _name = name;
    }


    /** Get the name of this trigger. */
    public String getName()
    {
        return _name;
    }

    /** Return the PACK policy for this trigger, default as empty. */
    public String getPackPolicy()
    {
        return "<pack/>";
    }


    /**
     *  Check whether the new event satisfy this trigger's condition. If 
     *  yes, then this function returns true.  The subclass must also
     *  encapsulate all the information to take action in the object
     *  _actionObject, which will be used by Sysnort to call takeAction.
     */
    public abstract boolean checkCondition(Event evt)
        throws Exception;


    /**
     *  Get the object containing all necessary info to take an action.
     *  Once the object is returned, the original variable _actionObject
     *  is set to null for next cycle.
     */
    public final Object getActionObject()
    {
        Object ret = _actionObject;
        _actionObject = null;
        return ret;
    }
    

    /**
     *  When the checkCondition returns true, Sysnort retrieves the action
     *  object and may call takeAction immediately or defer to a later time.
     */
    public abstract void takeAction(Object actionObject)
        throws Exception;

}
