/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package campus.sysnort.trigger;


import serp.util.Options;
import java.util.Set;
import java.util.Date;
import java.util.HashSet;

import campus.syslog.SyslogConstants;
import campus.sysnort.Trigger;
import solar.api.Event;
import solar.util.EmailUtil;


/**
 *  A Sysnort trigger that sends email when a reported stolen/missing
 *  card is associated with the network.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: MissingCardTrigger.java,v 1.6 2003/09/08 23:50:35 glchen Exp $
 */
public class MissingCardTrigger 
    extends Trigger
    implements SyslogConstants
{
    protected Set _cards;
    protected String _smtp;
    protected String _emailto;

    
    //=======================================================================
    // Implementation of campus.sysnort.Trigger interface.
    //=======================================================================

    public void init(Options opts)
        throws Exception
    {
        String cards = opts.getProperty("cards");
        if (cards == null)
        {
            throw new IllegalArgumentException(
                "List of missing cards not specified.");
        }
        _cards = new HashSet();        
        String[] ca = cards.split(" ");
        for (int i=0; i<ca.length; i++)
        {
            _cards.add(ca[i]);
        }
        _smtp = opts.getProperty("smtp", "mail.cs.dartmouth.edu");
        _emailto = opts.getProperty("emailto", "glchen@cs.dartmouth.edu");
    }
    
    public boolean checkCondition(Event evt)
        throws Exception
    {
        boolean ret = false;
        String mac = (String) evt.getValue(EVT_CLIENTMAC);
        if (_cards.contains(mac))
        {
            _actionObject = evt;
            ret = true;
        }
        return ret;
    }

    public void takeAction(Object actionObject)
        throws Exception
    {
        Event evt = (Event) actionObject;
        long timestamp = Long.parseLong((String)evt.getValue(EVT_TIMESTAMP));
        String msg = (String)evt.getValue(EVT_CLIENTMAC) + " "
            + new Date(timestamp*1000) + " "
            + (String) evt.getValue(EVT_APNAME) + " "
            + (String) evt.getValue(EVT_ACTION) + " "
            + (String) evt.getValue(EVT_NOTE);
        
        System.out.println(this.getName() + ": " + msg);

        /*
        EmailUtil.sendmail(
            _smtp, 
            _emailto,
            "[SYSNORT] Missing Card Alert",
            actionObject.toString());
        */
    }
}
