/*************************************************************************

"FreePastry" Peer-to-Peer Application Development Substrate

Copyright 2002, Rice University. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither  the name  of Rice  University (RICE) nor  the names  of its
contributors may be  used to endorse or promote  products derived from
this software without specific prior written permission.

This software is provided by RICE and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall RICE or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

********************************************************************************/

package rice.pastry.wire;


import rice.pastry.*;
import rice.pastry.dist.*;
import rice.pastry.routing.*;
import rice.pastry.messaging.*;
import rice.pastry.wire.exception.*;
import rice.pastry.wire.messaging.datagram.*;
import rice.pastry.wire.messaging.socket.*;

import java.io.*;
import java.nio.*;
import java.nio.channels.*;
import java.nio.charset.*;
import java.util.*;
import java.net.*;


/**
 *  Node handle that use TCP to transmit messages. Notice the call to
 *  receiveMessageImpl may block, see the modification to SocketChannelWriter
 *  class where we restrict its internal queue be 1 and the enqueue
 *  method may block as well.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: TcpNodeHandle.java,v 1.2 2003/09/20 21:35:53 glchen Exp $
 */
public class TcpNodeHandle 
    extends WireNodeHandle
{
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public TcpNodeHandle(InetSocketAddress address, NodeId nid) 
    {
        super(address, nid);
        debug("creating Socket handle for node: " + nid + " address: " + address);
    }

    public TcpNodeHandle(InetSocketAddress address, NodeId nid, PastryNode pn) 
    {
        super(address, nid, pn);
        debug("creating Socket handle for node: " + nid + " address: " + address);
        setLocalNode(pn);
    }


  /**
   * Called to send a message to the node corresponding to this handle.
   *
   * @param msg Message to be delivered, may or may not be routeMessage.
   */
  /**
   * Called to send a message to the node corresponding to this handle.
   *
   * @param msg Message to be delivered, may or may not be routeMessage.
   */
  public void receiveMessageImpl(Message msg) {
    assertLocalNode();

    WirePastryNode spn = (WirePastryNode) getLocalNode();

    if (isLocal) {
      debug("Sending message " + msg + " locally");
      spn.receiveMessage(msg);
    } else {
      debug("Passing message " + msg + " to the socket controller for writing (state == " + state + ")");

      // check to see if socket is open
      if (state != STATE_USING_TCP) {
        try {
          if (state == STATE_USING_UDP) {
              //glchen_edit, directly open socket connection
              LinkedList list = new LinkedList();
              list.addFirst(new SocketTransportMessage(msg, nodeId));
              connectToRemoteNode(list);
          } else {
            // if we're waiting to disconnect, send message over UDP anyway
            ((WirePastryNode) getLocalNode()).getDatagramManager().write(nodeId, address, msg);
          }
        } catch (Exception ex) {
          System.out.println(ex);
        }
      } else {
        writer.enqueue(new SocketTransportMessage(msg, nodeId));

        if (((WirePastryNode) getLocalNode()).inThread()) {
          key.interestOps(key.interestOps() | SelectionKey.OP_WRITE);
        } else {
          ((WirePastryNode) getLocalNode()).getSelectorManager().getSelector().wakeup();
        }

        debug("Enqueued message " + msg + " for writing in socket writer.");
      }
    }
  }
}
