/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.dan;


import java.net.URL;
import java.net.URLClassLoader;
import java.util.Hashtable;
import java.util.Iterator;

import solar.api.Guid;
import solar.api.Request;
import solar.api.Address;
import solar.service.fap.Pipe;
import solar.service.directory.Name;
import solar.service.directory.NameRecord;


/**
 *  The Solar implementation of DAN service.
 *  
 *  We enforce that all the pipe setup requests involving a Solar client are 
 *  always issued from that client. The Planet never issues request to setup 
 *  a pipe to a Solar client. So we can be sure that the remote end for a 
 *  requesting pipe is always on the Planet.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: SolarDanService.java,v 1.3 2003/10/05 18:50:55 glchen Exp $
 */
public class SolarDanService 
    extends AbstractDanService
{
    //=======================================================================
    // Implementation of solar.service.Service interface.
    //=======================================================================
    
    public Object handleRequest(Request req)
      throws Exception
    {
        return super.handleRequest(req);
    }


    //=======================================================================
    // Implementation of solar.service.dan.DanService interface.
    //=======================================================================

    public void connect(int type, 
        Guid sguid, String outport, Guid dguid, String inport)
        throws DanException
    {
        boolean slocal = _nodeman.getLocal(sguid) != null;
        boolean dlocal = _nodeman.getLocal(dguid) != null;
        if (!slocal && !dlocal)
        {
            throw new DanException(
                "Either source of sink of the pipe has to be local.");
        }

        if (slocal && dlocal)
        {
            _pc.pconnect(type, sguid, outport, dguid, inport);
        }
        else if (slocal)
        {
            Address addr = _nodeman.getRemote(dguid);
            if (addr == null)
                throw new DanException("Nothing known about sink " + dguid);
            _pc.pconnect(type, sguid, outport, addr, dguid, inport, false);
        }
        else
        {
            Address addr = _nodeman.getRemote(sguid);
            if (addr == null)
                throw new DanException("Nothing known about source " + sguid);
            _pc.pconnect(type, addr, sguid, outport, dguid, inport, false);
        }
    }

}
