/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.directory;


import java.util.List;

import solar.api.Id;
import solar.service.Service;


/**
 *  Directory service handles naming and resource discovery.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: DirectoryService.java,v 1.4 2003/09/25 21:37:37 glchen Exp $
 */
public interface DirectoryService 
    extends Service
{
    /** Add a directory listener for the specified pattern. */
    public void addListener(Name pattern, DirectoryListener listener)
        throws DirectoryException;
        

    /** Remove a directory listener for the specified pattern. */
    public void removeListener(Name pattern, DirectoryListener listener)
        throws DirectoryException;


    /** Return a list of advertisement whose name matches the pattern. */
    public List lookup(Name pattern)
        throws DirectoryException;


    /**
     *  Advertise a name to the name space, with an app-specific object.
     *  @return    the previous object if there is any.
     */
    public Object advertise(Name name, Id guid, Object obj)
        throws DirectoryException;


    /**
     *  Unadvertise a name from the name space.
     *  @return    the previous object if there is any.
     */
    public Object unadvertise(Name name, Id guid)
        throws DirectoryException;


    /**
     *  Update a name advertisement in the name space. 
     *  @return    the previous object if there is any.
     */
    public Object update(Name oldname, Id guid, Name newname, Object obj)
        throws DirectoryException;
}
