/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.fap;


import java.util.List;
import java.util.Vector;

import solar.api.Id;
import solar.api.Event;
import solar.api.Request;
import solar.api.Address;
import solar.api.Attribute;
import solar.impl.SerializedEvent;
import solar.service.dispatch.DispatchService;
import solar.service.fap.comm.PipePushEvent;
import solar.service.fap.comm.PipePullRequest;
import solar.service.rpc.RpcService;
import solar.service.rpc.RpcException;


/**
 *  A direct pipe has no buffer inside.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: DirectPipe.java,v 1.10 2003/10/05 18:50:57 glchen Exp $
 */
public class DirectPipe 
    extends Pipe
    implements FapConstants
{
    protected RpcService _rpc;
    protected DispatchService _dispatch;
    protected String _fapName;    
    protected String _transportName;

    
    //=======================================================================
    // Constructors.
    //=======================================================================

    public DirectPipe(Id pid, int type, FapService fap)
    {
        super(pid, type);
        _fapName = fap.getName();
        _transportName = fap.getTransportService(_pid).getName();
        _dispatch = fap.getDispatchService();
        _rpc = fap.getRpcService(_pid);
        if (_rpc == null)
        {
            throw new IllegalArgumentException(
                "Failed to get RPC service for specified pipe ID " + pid);
        }
    }
    

    //=======================================================================
    // Implementation of solar.service.fap.Pipe interface.
    //=======================================================================

    public void pushData(Object data)
        throws FapException
    {
        if (_type == FAP_PULL)
            throw new FapException("Cannot push data to a pull pipe.");

        if (_sink == null)
            throw new FapException("Sink channel not attached to this pipe.");


        if (!_sink.isRemote())
        {
            _sink.pushData(data);   // direct push to local sink
        }
        else        
        {
            // retrieve remote address and pipe ID
            RemoteSinkChannel rs = (RemoteSinkChannel) _sink;
            Address raddr = rs.getRemoteAddress();
            Id rpid = rs.getRemotePipeId();
            // construct a wrapper event
            Event evt = new PipePushEvent(rpid, data);
            // send the event
            try
            {
                _dispatch.dispatchEvent(evt, raddr, _fapName, _transportName);
            }
            catch (Exception ex)
            {
                throw new FapException("Failed to push a remote pipe.", ex);
            }
        }
    }

    public Object pullData(Object query)
        throws FapException
    {
        if (_type == FAP_PUSH)
            throw new FapException("Cannot pull data from a push pipe.");

        if (_source == null)
            throw new FapException("Source channel not attached to this pipe.");
        
        if (!_source.isRemote())
        {
            return _source.pullData(query); // direct pull from local sink
        }
        else
        {
            // retrieve remote address and pipe ID
            RemoteSourceChannel rs = (RemoteSourceChannel) _source;
            Address raddr = rs.getRemoteAddress();
            Id rpid = rs.getRemotePipeId();

            // construct a rpc request
            Request request = new PipePullRequest(rpid, query);
            try
            {
                return _rpc.invoke(raddr, _fapName, request, 3000);
            }
            catch (RpcException re)
            {
                throw new FapException("Failed to pull a remote pipe.", re);
            }
        }
    }
}
