/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.fap;


/**
 *  The data source of this channel is on the local site.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: LocalSourceChannel.java,v 1.5 2003/07/30 19:44:08 glchen Exp $
 */
public class LocalSourceChannel 
    implements SourceChannel
{
    protected DataSource _source;
    protected OutputPort _outport;

    
    //=======================================================================
    // Constructors.
    //=======================================================================

    /** Constructs an empty source channel. */
    public LocalSourceChannel()
    {
    }

    
    /** Constructs a source channel wit specified data source attached. */
    public LocalSourceChannel(DataSource source, OutputPort outport)
    {
        _source = source;
        _outport = outport;
    }
    

    //=======================================================================
    // Implementation of solar.service.fap.SourceChannel interface.
    //=======================================================================

    public boolean isRemote()
    {
        return false;
    }

    public void attachDataSource(
        DataSource source, OutputPort outport)
        throws FapException
    {
        if (_source != null)
        {
            throw new FapException(
                "A data source has already been attached to this channel.");
        }
        _source = source;
        _outport = outport;
    }

    public DataSource getDataSource()
        throws FapException
    {
        return _source;
    }

    public OutputPort getOutputPort()
        throws FapException
    {
        return _outport;
    }

    public Object pullData(Object query)
        throws FapException
    {
        return _source.pullData(query, _outport);
    }
}
