/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */
 
package solar.service.fork;


import java.io.Serializable;
import java.util.Iterator;
import java.util.Hashtable;

import solar.api.Id;
import solar.api.Address;


/**
 *  A configuration of a fork task.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: Task.java,v 1.2 2003/10/08 22:53:22 glchen Exp $
 */
public class Task
    implements Serializable
{
    Id _taskId;
    Address _src;
    Address _dest;
    int _start;
    int _end;
    String _command;
    Hashtable _parameters = new Hashtable();

    boolean _finished;
    
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public Task()
    {
        _taskId = Id.makeId("fork-task-");
    }

    public Task(Id id)
    {
        _taskId = id;
    }

    //=======================================================================
    // Public interface methods.
    //=======================================================================

    public void setFinished(boolean finished)
    {
        _finished = finished;
    }

    public boolean isFinished()
    {
        return _finished;
    }
    
    public Id getTaskId()
    {
        return _taskId;
    }

    public void setDestination(Address dest)
    {
        _dest = dest;
    }

    public Address getDestination()
    {
        return _dest;
    }

    public void setSource(Address src)
    {
        _src = src;
    }

    public Address getSource()
    {
        return _src;
    }

    public void setStartTime(int start)
    {
        _start = start;
    }

    public int getStartTime()
    {
        return _start;
    }

    public void setEndTime(int end)
    {
        _end = end;
    }

    public int getEndTime()
    {
        return _end;
    }
    
    public void setCommand(String command)
    {
        _command = command;
    }

    public void setParameter(String para, String value)
    {
        _parameters.put(para, value);
    }

    public boolean hasParameter(String para)
    {
        return _parameters.get(para) != null;
    }

    public String getFullCommand()
    {
        StringBuffer ret = new StringBuffer();
        ret.append(_command + " ");
        Iterator itr = _parameters.keySet().iterator();
        while (itr.hasNext())
        {
            String para = (String) itr.next();
            String value = (String) _parameters.get(para);
            ret.append("-" + para + " " + value + " ");
        }
        return ret.toString();
    }

    public String toString()
    {
        return _taskId + ": " + getFullCommand();
    }
}
