/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.multicast;


import org.apache.log4j.Logger;
import java.io.IOException;
import java.io.Serializable;

import solar.api.Id;
import solar.api.Event;
import solar.api.Request;
import solar.api.Address;
import solar.service.multicast.comm.ReceiveRequest;
import solar.service.rpc.RpcService;
import solar.service.rpc.RpcException;
import solar.service.rpc.RpcConstants;


/**
 *  A proxy class for remote multicast member. 
 *  
 *  @author     Guanling Chen
 *  @version    $Id: RemoteMulticastMember.java,v 1.5 2003/10/05 18:50:58 glchen Exp $
 */
public class RemoteMulticastMember 
    implements Serializable, MulticastMember, MulticastConstants, RpcConstants
{
    protected transient Logger _log;
    protected transient RpcService _rpc;
    protected transient String _sname;
    
    protected Id _id;
    protected Address _address;


    //===================================================================
    // Constructors.
    //===================================================================
    
    public RemoteMulticastMember(Id id, Address address)
    {
        _id = id;
        _address = address;
    }


    //===================================================================
    // Public interface methods.
    //===================================================================

    public void setRpcService(RpcService rpc, String sname)
    {
        _rpc = rpc;
        _sname = sname;
    }

    public void setLogger(Logger log)
    {
        _log = log;
    }


    //===================================================================
    // Implementation of solar.service.multicast.MulticastMember interface.
    //===================================================================
        
    public void handleMulticastEvent(Event evt)
    {
        try
        {
            Request req = new ReceiveRequest(_id, evt);
            _rpc.invoke(_address, _sname, req, RPC_TIMEOUT);
        }
        catch (RpcException re)
        {
            _log.error(re);
            re.printStackTrace();
        }
    }
}
