/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.pack;


import java.util.Hashtable;

import solar.api.Id;
import solar.api.Address;
import solar.service.pubsub.EndPoint;


/**
 *  A class manages all known end points and their related information.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: EndPointManager.java,v 1.5 2003/10/08 02:10:46 glchen Exp $
 */
public class EndPointManager 
{
    class EP
    {
        Id _session;
        Address _address;
        EndPoint _endpoint;
        EP(Id session, Address address, EndPoint endpoint)
        {
            _session = session;
            _address = address;
            _endpoint = endpoint;
        }
    }

    protected Hashtable _endpoints;
    
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public EndPointManager()
    {
        _endpoints = new Hashtable();
    }

    //=======================================================================
    // Public interface methods.
    //=======================================================================

    public synchronized boolean contains(Id id)
    {
        return _endpoints.get(id) != null;
    }
    
    public synchronized void attach(Id id,
        Id session, Address address, EndPoint endpoint)
    {
        _endpoints.put(id, new EP(session, address, endpoint));
    }

    public synchronized void detach(Id id)
    {
        _endpoints.remove(id);
    }

    public synchronized Id getSession(Id id)
    {
        Id ret = null;
        EP ep = (EP) _endpoints.get(id);
        if (ep != null)
        {
            ret = ep._session;
        }
        return ret;
    }

    public synchronized Address getAddress(Id id)
    {
        Address ret = null;
        EP ep = (EP) _endpoints.get(id);
        if (ep != null)
        {
            ret = ep._address;
        }
        return ret;
    }
    
    public synchronized EndPoint getEndPoint(Id id)
    {
        EndPoint ret = null;
        EP ep = (EP) _endpoints.get(id);
        if (ep != null)
        {
            ret = ep._endpoint;
        }
        return ret;
    }

    public synchronized void assertAttached(Id id)
    {
        if (_endpoints.get(id) == null)
        {
            throw new 
                IllegalStateException("Endpoint " + id + " not attached.");
        }
    }

    public synchronized void assertNotAttached(Id id)
    {
        if (_endpoints.get(id) != null)
        {
            throw new 
                IllegalStateException("Endpoint " + id + " already attached.");
        }
    }
}
