/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.transport;


import java.io.Serializable;
import java.io.IOException;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import solar.api.Id;
import solar.api.Address;
import solar.api.ParsingException;
import solar.SolarConstants;


/**
 *  The header for each ADU sent or received.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: Header.java,v 1.2 2003/10/05 18:51:02 glchen Exp $
 */
public class Header 
    implements Serializable, SolarConstants
{
    private static final long serialVersionUID = HEADER_SVID;
    
    protected Id _multiplex;
    protected Address _src;
    protected Address _dest;


    //=======================================================================
    // Constructors.
    //=======================================================================

    public Header()
    {
    }
    
    public Header(Id multiplex)
    {
        _multiplex = multiplex;
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================

    public Id getMultiplexId()
    {
        return _multiplex;
    }

    public void setMultiplexId(Id multiplex)
    {
        _multiplex = multiplex;
    }
    
    public Address getDestination()
    {
        return _dest;
    }

    public void setDestination(Address dest)
    {
        _dest = dest;
    }

    public Address getSource()
    {
        return _src;
    }

    public void setSource(Address src)
    {
        _src = src;
    }

    public String toString()
    {
        return "from " + _src.toString() + " to " + _dest.toString();
    }

    public void write(DataOutputStream out)
        throws IOException
    {
        byte[] data = _dest.toBytes();
        out.writeInt(data.length);
        out.write(data, 0, data.length);
        data = _src.toBytes();
        out.writeInt(data.length);
        out.write(data, 0, data.length);
        if (_multiplex == null)
        {
            out.writeInt(0);
            return;
        }
        data = _multiplex.toBytes();
        out.writeInt(data.length);
        out.write(data, 0, data.length);
    }

    public void read(DataInputStream in)
        throws IOException
    {
        try
        {
            // read destination address
            int len = in.readInt();
            byte[] data = new byte[len];
            in.readFully(data);
            _dest = AddressFactory.parseAddress(data);
            // read source address        
            len = in.readInt();
            data = new byte[len];
            in.readFully(data);
            _src = AddressFactory.parseAddress(data);
            // read multiplex ID if there is any
            len = in.readInt();
            if (len == 0)
            {
                return;
            }
            data = new byte[len];
            in.readFully(data);
            _multiplex = new Id(data);
        }
        catch (ParsingException pe)
        {
            throw new IOException(pe.toString());
        }
    }

    //=======================================================================
    // Serialize and Deserialize the header.
    //=======================================================================
    
    public void write(ObjectOutputStream out)
        throws IOException
    {
        out.writeObject(_dest);
        out.writeObject(_src);
        out.writeObject(_multiplex);
    }

    public void read(ObjectInputStream in)
        throws IOException, ClassNotFoundException
    {
        _dest = (Address) in.readObject();
        _src = (Address) in.readObject();
        _multiplex = (Id) in.readObject();
    }
}
