/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.service.transport;


import java.net.InetAddress;
import java.net.InetSocketAddress;

import solar.api.Address;
import solar.util.NetUtil;


/**
 *  Network address identified by IP and a port number.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: SocketAddress.java,v 1.12 2003/10/05 18:51:03 glchen Exp $
 */
public class SocketAddress 
    implements Address      // solar.transport.Address
{
    private InetAddress     _inetAddress;
    private int             _port;
    protected String        _string;
    protected int           _hashcode;


    //=======================================================================
    //  Constructors.
    //=======================================================================

    /** Constructs a socket address with a host name and port number. */
    public SocketAddress(String host, int port)
        throws Exception
    {
        _inetAddress = NetUtil.getInetAddress(host);
        _port = port;
        construction();
    }


    /** Constructs a socket address with a inet address and port number. */
    public SocketAddress(InetAddress addr, int port)
        throws Exception
    {
        _inetAddress = addr;
        _port = port;
        construction();
    }

    /** Constructs a socket address with a inet socket address. */
    public SocketAddress(InetSocketAddress saddr)
    {
        _inetAddress = saddr.getAddress();
        _port = saddr.getPort();
        construction();
    }

    
    /** Constructs a socket address with ip_address:port_number argument. */
    public SocketAddress(String pair)
        throws Exception
    {
        int p = pair.indexOf(":");
        if (p == -1)
            throw new Exception("port number not specified.");
        _inetAddress = NetUtil.getInetAddress(pair.substring(0, p));
        _port = Integer.parseInt(pair.substring(p+1));
        construction();
    }


    /** common constructor operations. */
    private void construction()
    {
        _string = NetUtil.getIpAddress(_inetAddress) + ":" + _port;
        _hashcode = _inetAddress.hashCode() ^ _port << 8;
    }


    //=======================================================================
    //  Implementation of solar.service.transport.Address interface.
    //=======================================================================

    public String toString()
    {
        return _string;
    }

    public byte[] toBytes()
    {
        byte[] cnt = toString().getBytes();
        byte[] ret = new byte[cnt.length + 1];
        ret[0] = (byte) getEncoding();
        System.arraycopy(cnt, 0, ret, 1, cnt.length);
        return ret;
    }

    public int getEncoding()
    {
        return TransportConstants.SOCKET_ADDRESS;
    }

    public boolean equals(Object obj)
    {
        if (obj == null || ! (obj instanceof SocketAddress))
            return false;

        SocketAddress other = (SocketAddress) obj;
        return _string.equals(other._string);
    }

    public int hashCode()
    {
        return _hashcode;
    }

    
    //=======================================================================
    //  Public interface methods.
    //=======================================================================

    /** Return IP address. */
    public InetAddress getInetAddress()
    {
        return _inetAddress;
    }

    /** Return port number. */
    public int getPort()
    {
        return _port;
    }
}
