/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.test;


import serp.util.Options;
import java.util.List;
import java.util.Random;

import solar.api.Event;
import solar.api.Request;
import solar.api.Attribute;
import solar.impl.PlainEvent;
import solar.service.dan.Operator;


/**
 *  This operator mutates the received events by adding random attributes.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: MutateOperator.java,v 1.6 2003/09/19 00:38:31 glchen Exp $
 */
public class MutateOperator 
    extends Operator
{
    String[] array = {"Google", "Microsoft", "IBM", "Intel", "HP"};
    
    Random _ran;

    
    //=======================================================================
    // Implementation of solar.service.dan.Operator interface.
    //=======================================================================

    public void init(Options opts)
    {
        long seed = opts.getLongProperty("seed", System.currentTimeMillis());
        _ran = new Random(seed);
    }
    
    public void handleEvent(Event evt)
        throws Exception
    {
        List attrs = evt.getAttributes();
        int idx = _ran.nextInt();
        idx = (idx < 0) ? -idx : idx;
        idx %= array.length;
        Attribute attr = new Attribute(array[idx], "cool place");
        attrs.add(attr);
        Event nevt = new PlainEvent(attrs);
        publish(evt);
    }

    public Object handleQuery(Request query)
        throws Exception
    {
        return null;
    }
}
