/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.util;


import java.util.Date;
import java.util.Properties;
import javax.mail.*;
import javax.mail.internet.*;


/**
 *  Contains common actions using email system.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: EmailUtil.java,v 1.1 2003/09/07 05:05:46 glchen Exp $
 */
public class EmailUtil
{
    //=======================================================================
    // Public interface methods.
    //=======================================================================

    /** Basic sendmail function, multiple receivers separated by space. */
    public static void sendmail(String smtp, String receiver, 
        String subject, String text)
        throws Exception
    {
        // set smtp server
        Properties props = new Properties();
        props.put("mail.smtp.host", smtp);
        
        // set session properties
        Session session = Session.getDefaultInstance(props, null);
        session.setDebug(false);
        
        // set email message header
        Message msg = new MimeMessage(session);
        msg.setFrom(new InternetAddress("sysnort-noreply@cs.dartmouth.edu"));
        String[] rcv = receiver.split(" ");
        InternetAddress[] address = new InternetAddress[rcv.length];
        for (int i=0; i<address.length; i++)
            address[i] = new InternetAddress(rcv[i]); 
        msg.setRecipients(Message.RecipientType.TO, address);
        
        // set email subject and body
        msg.setSubject(subject);
        msg.setSentDate(new Date());
        // If the desired charset is known, you can use
        // setText(text, charset)
        msg.setText(text);
        
        // send the email message
        Transport.send(msg);
    }
}
