/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.util;


/**
 *  A not synchronized non-blocking queue based on a fast linked-list.
 *
 *  @author     Guanling Chen
 *  @version    $Id: FastUnsyncQueue.java,v 1.3 2003/09/10 01:08:10 glchen Exp $
 */
public class FastUnsyncQueue
    implements Queue
{
    protected ssLinkedList _list;
    protected int _size;


    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public FastUnsyncQueue()
    {
        _size = 0;
        _list = new ssLinkedList();
    }


    //=======================================================================
    // Implementation of solar.util.Queue interface.
    //=======================================================================
    
    public int size()
    {
        return _size;
    }


    public void enqueue(Object item)
    {
        _size ++;
        _list.add_to_tail(item);
//        notifyAll();
    }


    public void enqueueAll(Object[] items)
    {
        int len = items.length;
        for (int i=0; i<len; i++)
        {
            _list.add_to_tail(items[i]);
        }
        _size += len;
//        notifyAll();
    }


    public Object dequeue()
        throws InterruptedException
    {
/*        if (_size == 0)
        {
            wait();
        }*/
        Object ret = null;
        if (_size > 0)
        {
            _size --;
            ret = (Object) _list.remove_head();
        }
        return ret;
    }


    public Object dequeue(long timeout)
        throws InterruptedException
    {
/*        if (_size == 0)
        {
            wait(timeout);
        }*/
        Object ret = null;
        if (_size > 0)
        {
            _size --;
            ret = (Object) _list.remove_head();
        }
        return ret;

    }


    public Object[] dequeueAll()
    {
        if (_size == 0)
        {
            return null;
        }

        Object[] ret = new Object[_size];
        for (int i=0; i<_size; i++)
        {
            ret[i] = (Object) _list.remove_head();
        }
        _size = 0;
        return ret;
    }
}
