/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package solar.util;


import java.net.URL;
import java.net.URLClassLoader;
import java.net.InetAddress;
import java.net.UnknownHostException;


/**
 *  Some utility methods about networking.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: NetUtil.java,v 1.8 2003/09/30 15:53:38 glchen Exp $
 */
public class NetUtil 
{
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    /** Prevent object initialization. */
    private NetUtil()
    {
    }


    //=======================================================================
    // Public interface methods.
    //=======================================================================

    /** Get the InetAddress from host name, special treat on localhost. */
    public static InetAddress getInetAddress(String name)
        throws UnknownHostException
    {
        InetAddress ret = null;
        if (name.equals("localhost"))
        {
            try
            {
                ret = InetAddress.getLocalHost();
            }
            catch (UnknownHostException uhe)
            {
                ret = InetAddress.getByName("localhost");
            }
            return ret;
        }
        InetAddress[] ias = InetAddress.getAllByName(name);
        if (ias.length == 1)
        {
            return ias[0];
        }
        else
        {
            for (int i=0; i<ias.length; i++)
            {
                ret = ias[i];
                if (ret.toString().indexOf("127.0.0.1") == -1)
                {
                    break;
                }
            }
            return ret;
        }
    }

    /** Get the digital IP address (no host name) in string format. */
    public static String getIpAddress(InetAddress inetAddress)
    {
        if (inetAddress == null)
        {
            throw new IllegalArgumentException("Null address specified");
        }
        String ret = inetAddress.toString();
        int pos = ret.indexOf("/");
        if (pos != -1)
        {
            ret = ret.substring(pos + 1);
        }
        return ret;
    }


    /** Create a new object instance of specified class. */
    public static Object loadObject(String codebase, String clazz)
        throws Exception
    {
        if (codebase == null || codebase.equals(""))
            codebase = "http://localhost/";
        URL urls[] = { new URL(codebase) };
        URLClassLoader loader = new URLClassLoader(urls);
        Class c = loader.loadClass(clazz);
        Object obj = c.newInstance();
        return obj;
    }
}
