/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package versus.meeting;


import serp.util.Options;
import org.apache.log4j.Logger;
import org.apache.log4j.Level;
import org.apache.log4j.BasicConfigurator;
import java.io.File;
import java.util.Date;
import java.util.Properties;
import javax.mail.*;
import javax.mail.internet.*;

import solar.api.Event;
import solar.service.dan.Application;
import solar.util.FileUtil;
import solar.Client;


/**
 *  This application simply receives events from MeetingDetector and notify
 *  some users by email.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: MeetingApp.java,v 1.3 2003/09/19 00:38:32 glchen Exp $
 */
public class MeetingApp
    extends Application
{
    private static Logger _log  = Logger.getLogger("solar");
    
    private long started = 0;

    //=======================================================================
    // Implementation of solar.service.dan.Application interface.
    //=======================================================================
    
    public void handleEvent(Event evt)
        throws Exception
    {
        boolean meeting = "true".equals(evt.getValue("meeting"));
        String room = (String) evt.getValue("room");
        long timestamp = 
            Long.parseLong((String) evt.getValue("timestamp"));
        if (meeting)
        {
            started = timestamp;
        }
        else
        {
            //notify
            Properties props = new Properties();
            props.put("mail.smtp.host", "cs.dartmouth.edu");

            Session session = Session.getDefaultInstance(props, null);
            session.setDebug(false);

            Message msg = new MimeMessage(session);
            msg.setFrom(new InternetAddress("glchen@cs.dartmouth.edu"));
            InternetAddress[] address = {
                new InternetAddress("ubichen@cs.dartmouth.edu"), 
                new InternetAddress("glchen@cs.dartmouth.edu") };

            msg.setRecipients(Message.RecipientType.TO, address);
            msg.setSubject("116 Meeting Digest");
            msg.setSentDate(new Date());
            // If the desired charset is known, you can use
            // setText(text, charset)
            String msgText = "\n\n"
                + "started\t- " + new Date(started).toString() + "\n"
                + "ended  \t- " + new Date(timestamp).toString() + "\n"
                + "lasted \t- " + new Long((timestamp-started)/60000) + " minutes.\n\n";
            msg.setText(msgText);
            Transport.send(msg);
            System.out.print(new Date(started) + " - " + new Date(timestamp));
            System.out.print(" - ");
            System.out.print((timestamp - started)/60000);
            System.out.println(" minutes.");
        }
    }
    

    //=======================================================================
    // Implementation of solar.service.dan.Application interface.
    //=======================================================================

    public static void main(String[] args)
        throws Exception
    {
        Options opts = new Options();
        opts.setFromCmdLine(args);
        if (opts.size() == 0)
        {
            String usage = "command line usage: \n"
                + "\t-debug Whether to turn on debug info. \n"
                + "\t-planet The hostname of a Planet it connects to.\n"
                + "\t-port The port number of the Planet. \n"
                + "\t-subfile The file containting solar subscription. \n"
                + "\t-root The operator to subscribe in the graph spec. \n";
            System.out.println(usage);
            System.exit(0);
        }

        // configure the logger
        boolean debug = opts.getBooleanProperty("debug", true);
        if (debug)
            _log.setLevel( (Level) Level.DEBUG );
        else
            _log.setLevel( (Level) Level.WARN );
        BasicConfigurator.configure();

        // configure the solar client
        String host = opts.getProperty("planet");
        int port = opts.getIntProperty("port", 5470);
        if (host == null)
        {
            throw new IllegalArgumentException("invalid arguments.");
        }
        Client client = new Client(host, port);

        // make the subscription
        MeetingApp app = new MeetingApp();
        String subfile = opts.getProperty("subfile", "sub.txt");
        String root = opts.getProperty("root", "root");
        client.subscribe(FileUtil.getContent(subfile), root, app);
    }
}
