/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package versus.meeting;


import serp.util.Options;
import java.io.*;
import java.util.*;

import solar.api.Event;
import solar.api.Request;
import solar.api.Attribute;
import solar.impl.EventFactory;
import solar.service.dan.Operator;
import versus.VersusMapping;


/**
 *  A primitive operator that detects the ongoing meeting based on badges attached
 *  to chairs.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: MeetingDetector.java,v 1.4 2003/09/19 00:38:32 glchen Exp $
 */
public class MeetingDetector 
    extends Operator
{
    private String _room;
    Set _movingChairs = new HashSet();
    boolean _meetingStarted = false;
    long _started;
    long[] _buffer = new long[4];
    boolean[] _status = new boolean[4];


    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public MeetingDetector()
    {
        _buffer[0] = _buffer[1] = _buffer[2] = _buffer[3] = 0;
        _status[0] = _status[1] = _status[2] = _status[3] = false;
    }


    //=======================================================================
    // Implementation of solar.service.dan.Operator interface.
    //=======================================================================
    
    public void init(Options opts)
    {
        // the two args should be room number
        // assuming the chair having names like room#/id
        _room = opts.getProperty("room", "116");
    }

    public Object handleQuery(Request query)
        throws Exception
    {
        return null;
    }

    public void handleEvent(Event evt)
        throws Exception
    {
        // assuming the events are from transformer
        // and will have name and zone attributes
        String name = (String) evt.getValue("name");
        if (name == null || !name.startsWith(_room))
            return;

        int idx = 0;
        if (name.equals("116/chair-south")) idx = 0;
        else if (name.equals("116/chair-north")) idx = 1;
        else if (name.equals("116/chair-west")) idx = 2;
        else if (name.equals("116/chair-east")) idx = 3;
        boolean motion = "true".equals(evt.getAttribute("motion"));
        if (motion)
        {
            if (!_status[idx])
            {
                _status[idx] = true;
                _buffer[idx] = 0;
            }
            else
            {
                _buffer[idx] ++;
                if (_buffer[idx]>2)
                {
                    _movingChairs.add(name);
                }
            }
        }
        else
        {
            if (_status[idx])
            {
                _status[idx] = false;
                _buffer[idx] = 0;
            }
            else
            {
                _buffer[idx] ++;
                if (_buffer[idx]>9)
                {
                    _movingChairs.remove(name);
                }
            }
        }

        long timestamp = 
            Long.parseLong((String) evt.getValue("timestamp"));
        Date now = new Date(timestamp);
        if (!_meetingStarted && _movingChairs.size()>1)
        {
            _meetingStarted = true;
            _started = timestamp;

            List at = new Vector();
            at.add(new Attribute("meeting", "true"));
            at.add(new Attribute("room", _room));
            at.add(new Attribute("timestamp", evt.getAttribute("timestamp")));
            Event newevt = EventFactory.getPlainEvent(at);
            publish(newevt);
        }
        else if (_meetingStarted && _movingChairs.size()<1)
        {
            _meetingStarted = false;
            
            List at = new Vector();
            at.add(new Attribute("meeting", "false"));
            at.add(new Attribute("room", _room));
            at.add(new Attribute("timestamp", evt.getAttribute("timestamp")));
            Event newevt = EventFactory.getPlainEvent(at);
            publish(newevt);
        }
    }


    //=======================================================================
    // Main entrance for test purpose.
    //=======================================================================

    public static void main(String[] args)
        throws Exception
    {
        Options opts = new Options();
        opts.setFromCmdLine(args);

        String _trace = opts.getProperty("trace", "versus.log");
        String _map = opts.getProperty("mapping", "versus.xml");
        String _room = opts.getProperty("room", "116");

        VersusMapping _mapping = new VersusMapping(_map);

        Set _movingChairs = new HashSet();
        boolean _meetingStarted = false;
        long _started = 0;
        long[] _buffer = new long[4];
        _buffer[0] = _buffer[1] = _buffer[2] = _buffer[3] = 0;
        boolean[] _status = new boolean[4];
        _status[0] = _status[1] = _status[2] = _status[3] = false;
        
        BufferedReader reader = 
            new BufferedReader(new FileReader(_trace));
        String line = reader.readLine();
        while (line != null)
        {
            Event evt = EventFactory.getPlainEvent(line);
            String badge = (String) evt.getValue("badge_num");
            String name = _mapping.getName(badge);
            if (name==null || !name.startsWith(_room))
            {
                line = reader.readLine();
                continue;
            }
            int idx = 0;
            if (name.equals("116/chair-south")) idx = 0;
            else if (name.equals("116/chair-north")) idx = 1;
            else if (name.equals("116/chair-west")) idx = 2;
            else if (name.equals("116/chair-east")) idx = 3;
            boolean motion = "true".equals(evt.getAttribute("motion"));
            if (motion)
            {
                if (!_status[idx])
                {
                    _status[idx] = true;
                    _buffer[idx] = 0;
                }
                else
                {
                    _buffer[idx] ++;
                    if (_buffer[idx]>2)
                    {
                        _movingChairs.add(name);
                    }
                }
            }
            else
            {
                if (_status[idx])
                {
                    _status[idx] = false;
                    _buffer[idx] = 0;
                }
                else
                {
                    _buffer[idx] ++;
                    if (_buffer[idx]>9)
                    {
                        _movingChairs.remove(name);
                    }
                }
            }

            long timestamp = 
                Long.parseLong((String) evt.getValue("timestamp"));
            Date now = new Date(timestamp);
            if (!_meetingStarted && _movingChairs.size()>1)
            {
                System.out.print("\nMeeting in " + _room + ": ");
                System.out.print(now);
                _meetingStarted = true;
                _started = timestamp;
            }
            else if (_meetingStarted && _movingChairs.size()<1)
            {
                System.out.print("\nMeeting in " + _room + ": ");
                System.out.print(now);
                System.out.print("\t for ");
                System.out.print( (int) ((timestamp-_started)/60000));
                System.out.println(" minutes.");
                _meetingStarted = false;
            }
            line = reader.readLine();
        }
    }
}
