/*
 * ============================================================================
 *                   The Solar Software License, Version 1.0
 * ============================================================================
 *
 *       Copyright (C) 2002-2003 Dartmouth College. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The  names  "Solar"  and   "Dartmouth  College"  must  not  be  used   to 
 *    endorse  or   promote   products  derived   from this  software   without
 *    prior  written   permission.   For  written   permission,  please contact 
 *    solar@cs.dartmouth.edu.
 * 
 * 4. Products  derived from this software may not  be called "Solar", nor may
 *    "Solar" appear  in their name,  without prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * DARTMOUTH COLLEGE OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY  DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR   CONSEQUENTIAL   DAMAGES   (INCLUDING
 * BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE  GOODS  OR  SERVICES;  LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the Solar project at Dartmouth College. For more information on
 * the Solar project, please see <http://www.cs.dartmouth.edu/~solar/>.

 */

package versus.meeting;


import java.util.Hashtable;


/**
 *  Implementation of meeting world for reinforcement learning environment.
 *  
 *  @author     Guanling Chen
 *  @version    $Id: RLMeetingWorld.java,v 1.8 2003/08/01 00:39:49 glchen Exp $
 */
public class RLMeetingWorld 
    implements RLWorld
{
	static final int NUM_OBJECTS    = 2;    // two state variables
    static final int NUM_ACTIONS    = 2;    // meeting or no meeting
	static final double INIT_VALS   = 0;
    static final int NUM_STATES     = 42;
    
    RLMeetingAgent _agent;
    
    int[] _state = new int[NUM_OBJECTS];  // 2 state variable
    boolean _endState = false;
    
//    int _reward = 0;
    int _action = -1;
    int _actionTime = 0;
    int _holding = 0;
    int _numMoving = -1;    // num of moving badges
    int _timeMoving = 0;       // starting time of previous state
    int _currentTime = 0;
    
    
    //=======================================================================
    // Constructors.
    //=======================================================================
    
    public RLMeetingWorld(RLMeetingAgent agent)
    {
        _agent = agent;
        resetState();
    }


    //=======================================================================
    // Implementation of versus.meeting.RLWorld interface.
    //=======================================================================

    public int[] getDimension()
    {
		int[] retDim = new int[NUM_OBJECTS + 1];
		retDim[0] = 5;  // 0-4 badges can move
        retDim[1] = NUM_STATES; // max sum of sigmoids is 40
		retDim[NUM_OBJECTS] = NUM_ACTIONS;
		return retDim;
    }
    
    public int[] getNextState( int action )
    {
        int[] evt = _agent.getNextEvent();
        
        int time = evt[0];
        _state[0] = evt[1];
        _state[1] = evt[2];

        _currentTime = time;
        if (_state[0] != _numMoving)
        {
            _numMoving = _state[0];
            _timeMoving = _currentTime;
        }
        if (action != _action)
        {
            _action = action;
            _holding = _currentTime - _actionTime;
            _actionTime = action;
        }
        else
        {
            _holding = 0;
        }
        return _state;
    }
    
    public double getReward()
    {
        int ret = 0;
        if (_numMoving == 0
            && _currentTime - _timeMoving > 250)
        {
            ret = (_action == 0) ? 100 : -100;
        }
        if (_numMoving > 0
            && _currentTime - _timeMoving > 50)
        {
            ret = (_action == 0) ? -100 : 100;
        }
        if (_holding > 0 && _holding < 300)
        {
            ret = -1000000;
        }
        return ret;
    }
    
    public boolean validAction( int action )
    {
        return true;
    }
    
    public boolean endState()
    {
        return _endState;
    }

    public void setEndState(boolean end)
    {
        _endState = end;
    }

    public int[] resetState()
    {
//        _reward = 0;
        return _state;
    }

    public double getInitValues()
    {
        return INIT_VALS;
    }
}
